<?php

namespace Drupal\user_hash\Plugin\Action;

use Drupal\Core\Action\ActionBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Component\Utility\Crypt;

/**
 * Generate user hashes.
 *
 * @Action(
 *   id = "user_generate_user_hash_action",
 *   label = @Translation("Generate hash for the selected user(s)"),
 *   type = "user"
 * )
 */
class GenerateUserHash extends ActionBase {

  protected $cacheService;

  /**
   * Class constructor.
   */
  public function __construct($cacheService) {
    $this->cacheService = $cacheService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cache.cache')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function execute($account = NULL) {
    if ($account !== FALSE) {
      $config = \Drupal::config('user_hash.settings');
      $algorithm = $config->get('algorithm');
      $random_bytes = $config->get('random_bytes');

      $hash = self::generateHash($algorithm, $random_bytes);
      \Drupal::service('user.data')->set('user_hash', $account->id(), 'hash', $hash);
      Cache::invalidateTags(array('user:' . $account->id()));
    }
  }
  
  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    /** @var \Drupal\user\UserInterface $object */
    $access = $object->status->access('edit', $account, TRUE)
      ->andIf($object->access('update', $account, TRUE));

    return $return_as_object ? $access : $access->isAllowed();
  }

  /**
   * Generate hash.
   * 
   * @param string $algorithm
   *   PHP hash algorithm to use.
   * @param string $count
   *   Quantity of characters to use for the random value at hash generation.
   * @param bool $raw_output
   *   When set to TRUE, outputs raw binary data. FALSE outputs lowercase hexits.
   *
   * @return string
   *   Returns the generated hash.
   */
  public static function generateHash($algorithm, $count, $raw_output = FALSE) {
    return hash($algorithm, Crypt::randomBytes($count), $raw_output);
  }

}
